@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid = "$Id: plot2d.c,v 1.53 1998/06/18 14:55:14 ddenholm Exp $";
+static char *RCSid = "$Id: plot2d.c,v 1.16.2.12 2002/01/31 21:18:22 lhecking Exp $";
 #endif
 
 /* GNUPLOT - plot2d.c */
@@ -248,10 +248,9 @@ void plotrequest()
     }				/* first '[' */
     if (parametric || polar)	/* set optional x ranges */
 	LOAD_RANGE(FIRST_X_AXIS);
-    else {
-	/* order of t doesn't matter, but x does */
-	CHECK_REVERSE(FIRST_X_AXIS);
-    }
+
+    /* order of x range does matter, even if we're in parametric mode */
+    CHECK_REVERSE(FIRST_X_AXIS);
 
     LOAD_RANGE(FIRST_Y_AXIS);
     CHECK_REVERSE(FIRST_Y_AXIS);
@@ -288,7 +287,8 @@ struct curve_points *this_plot;
  * it will later be moved passed title/with/linetype/pointtype
  */
 {
-    register int i /* num. points ! */ , j, col;
+    int i /* num. points ! */ , j;
+    int max_cols, min_cols;	/* allowed range of column numbers */
     double v[NCOL];
     int storetoken = this_plot->token;
 
@@ -297,38 +297,52 @@ struct curve_points *this_plot;
     switch (this_plot->plot_style) {	/* set maximum columns to scan */
     case XYERRORBARS:
     case BOXXYERROR:
-	col = 7;
+	max_cols = 7;
+	min_cols = 4;
+	break;
+
+    case FINANCEBARS:
+    case CANDLESTICKS:
+	min_cols = max_cols = 5;
 	break;
 
     case BOXERROR:
-    case FINANCEBARS:
-    case CANDLESTICKS:
-	col = 5;
+	max_cols = 5;
+	min_cols = 4;
+	break;
+
+    case VECTOR:
+	min_cols = max_cols = 4;
 	break;
 
     case XERRORBARS:
     case YERRORBARS:
-    case VECTOR:
-	col = 4;
+	max_cols = 4;
+	min_cols = 3;
 	break;
 
     case BOXES:
-	col = 4;
+	min_cols = 2;
+	max_cols = 4;
 	break;
 
     default:
-	col = 2;
+	min_cols = 1;
+	max_cols = 2;
     }
 
     if (this_plot->plot_smooth == ACSPLINES)
-	col = 3;
+	max_cols = 3;
 
-    if (df_no_use_specs > col)
-	fputs("warning : too many using specs for this style\n", stderr);
+    if (df_no_use_specs > max_cols)
+	int_error("Too many using specs for this style", NO_CARET);
+
+    if (df_no_use_specs >0 && df_no_use_specs < min_cols)
+	int_error("Not enough columns for this style", NO_CARET);
 
     i = 0;
-    while ((j = df_readline(v, col)) != DF_EOF) {
-	/* j <= col */
+    while ((j = df_readline(v, max_cols)) != DF_EOF) {
+	/* j <= max_cols */
 
 	if (i >= this_plot->p_max) {
 	    /*
@@ -704,23 +718,87 @@ struct curve_points *this_plot;
 int plot_num;
 {
     int i, curve;
+    char *table_format = NULL;
+    /* The data format is determined by the format of the axis labels.
+     * See 'set format'.  Patch by Don Taber
+     */
+    table_format = gp_alloc(strlen(xformat)+strlen(yformat)+5, "table format");
+    strcpy(table_format, xformat);
+    strcat(table_format, " ");
+    strcat(table_format, yformat);
+    strcat(table_format, " %c\n");
 
+    /* Not sure whether the missing plot styles require special treatment.
+     * They all fall under the "default" case right now. Lars
+     */
     for (curve = 0; curve < plot_num;
 	 curve++, this_plot = this_plot->next_cp) {
-	fprintf(gpoutfile, "#Curve %d, %d points\n#x y type\n", curve, this_plot->p_count);
+	fprintf(gpoutfile, "#Curve %d, %d points\n#x y", curve, this_plot->p_count);
+	switch (this_plot->plot_style) {
+	case BOXES:
+	case XERRORBARS:
+	    fprintf(gpoutfile, " xlow xhigh");
+	    break;
+	case BOXERROR:
+	case YERRORBARS:
+	    fprintf(gpoutfile, " ylow yhigh");
+	    break;
+	case BOXXYERROR:
+	case XYERRORBARS:
+	    fprintf(gpoutfile, " xlow xhigh ylow yhigh");
+	    break;
+	case FINANCEBARS:
+	case CANDLESTICKS:
+	default:
+	    /* ? */
+	    break;
+	}
+
+	fprintf(gpoutfile, " type\n");
 	for (i = 0; i < this_plot->p_count; i++) {
-	    fprintf(gpoutfile, "%g %g %c\n",
+	    fprintf(gpoutfile, "%g %g",
 		    this_plot->points[i].x,
-		    this_plot->points[i].y,
+		    this_plot->points[i].y);
+	    switch (this_plot->plot_style) {
+	    case BOXES:
+	    case XERRORBARS:
+		fprintf(gpoutfile, " %g %g",
+			this_plot->points[i].xlow,
+			this_plot->points[i].xhigh);
+		break;
+	    case BOXERROR:
+	    case YERRORBARS:
+		fprintf(gpoutfile, " %g %g",
+			this_plot->points[i].ylow,
+			this_plot->points[i].yhigh);
+		break;
+	    case BOXXYERROR:
+	    case XYERRORBARS:
+		fprintf(gpoutfile, " %g %g %g %g",
+			this_plot->points[i].xlow,
+			this_plot->points[i].xhigh,
+			this_plot->points[i].ylow,
+			this_plot->points[i].yhigh);
+		break;
+	    case FINANCEBARS:
+	    case CANDLESTICKS:
+	    default:
+		/* ? */
+                break;
+	    }
+	    fprintf(gpoutfile, " %c\n",
 		    this_plot->points[i].type == INRANGE ? 'i'
 		    : this_plot->points[i].type == OUTRANGE ? 'o'
 		    : 'u');
 	}
 	fputc('\n', gpoutfile);
     }
-/* two blank lines between plots in table output */
+
+    /* two blank lines between plots in table output */
     fputc('\n', gpoutfile);
     fflush(gpoutfile);
+    
+    free(table_format);
 }
 
 /*
@@ -1194,16 +1272,24 @@ do{ assert(!polar && !parametric); \
     if (plot_num == 0 || first_plot == NULL) {
 	int_error("no functions or data to plot", c_token);
     }
+
+
     if (uses_axis[FIRST_X_AXIS]) {
 	if (max_array[FIRST_X_AXIS] == -VERYLARGE ||
-	    min_array[FIRST_X_AXIS] == VERYLARGE)
+	    min_array[FIRST_X_AXIS] == VERYLARGE) {
+	    cp_free(first_plot);
+	    first_plot = NULL;
 	    int_error("all points undefined!", NO_CARET);
+	}
 	FIXUP_RANGE_FOR_LOG(FIRST_X_AXIS, x);
     }
     if (uses_axis[SECOND_X_AXIS]) {
 	if (max_array[SECOND_X_AXIS] == -VERYLARGE ||
-	    min_array[SECOND_X_AXIS] == VERYLARGE)
+	    min_array[SECOND_X_AXIS] == VERYLARGE) {
+	    cp_free(first_plot);
+	    first_plot = NULL;
 	    int_error("all points undefined!", NO_CARET);
+	}
 	FIXUP_RANGE_FOR_LOG(SECOND_X_AXIS, x2);
     } else {
 	assert(uses_axis[FIRST_X_AXIS]);
@@ -1211,8 +1297,9 @@ do{ assert(!polar && !parametric); \
 	    min_array[SECOND_X_AXIS] = min_array[FIRST_X_AXIS];
 	if (auto_array[SECOND_X_AXIS] & 2)
 	    max_array[SECOND_X_AXIS] = max_array[FIRST_X_AXIS];
+	if (! auto_array[SECOND_X_AXIS])
+	    FIXUP_RANGE_FOR_LOG(SECOND_X_AXIS, x2);
     }
-
     if (!uses_axis[FIRST_X_AXIS]) {
 	assert(uses_axis[SECOND_X_AXIS]);
 	if (auto_array[FIRST_X_AXIS] & 1)
@@ -1220,27 +1307,39 @@ do{ assert(!polar && !parametric); \
 	if (auto_array[FIRST_X_AXIS] & 2)
 	    max_array[FIRST_X_AXIS] = max_array[SECOND_X_AXIS];
     }
+
+
     if (uses_axis[FIRST_Y_AXIS]) {
 	if (max_array[FIRST_Y_AXIS] == -VERYLARGE ||
-	    min_array[FIRST_Y_AXIS] == VERYLARGE)
+	    min_array[FIRST_Y_AXIS] == VERYLARGE) {
+	    cp_free(first_plot);
+	    first_plot = NULL;
 	    int_error("all points undefined!", NO_CARET);
+	}
 	fixup_range(FIRST_Y_AXIS, "y");
 	FIXUP_RANGE_FOR_LOG(FIRST_Y_AXIS, y);
-    }				/* else we want to copy y2 range, but need to fix it up first */
+    }
     if (uses_axis[SECOND_Y_AXIS]) {
 	if (max_array[SECOND_Y_AXIS] == -VERYLARGE ||
-	    min_array[SECOND_Y_AXIS] == VERYLARGE)
+	    min_array[SECOND_Y_AXIS] == VERYLARGE) {
+	    cp_free(first_plot);
+	    first_plot = NULL;
 	    int_error("all points undefined!", NO_CARET);
+	}
 	fixup_range(SECOND_Y_AXIS, "y2");
 	FIXUP_RANGE_FOR_LOG(SECOND_Y_AXIS, y2);
     } else {
+	/* else we want to copy y2 range */
 	assert(uses_axis[FIRST_Y_AXIS]);
 	if (auto_array[SECOND_Y_AXIS] & 1)
 	    min_array[SECOND_Y_AXIS] = min_array[FIRST_Y_AXIS];
 	if (auto_array[SECOND_Y_AXIS] & 2)
 	    max_array[SECOND_Y_AXIS] = max_array[FIRST_Y_AXIS];
+	/* Log() fixup is only necessary if the range was *not* copied from
+	 * the (already logarithmized) yrange */
+	if (! auto_array[SECOND_Y_AXIS])
+	    FIXUP_RANGE_FOR_LOG(SECOND_Y_AXIS, y2);
     }
-
     if (!uses_axis[FIRST_Y_AXIS]) {
 	assert(uses_axis[SECOND_Y_AXIS]);
 	if (auto_array[FIRST_Y_AXIS] & 1)
@@ -1248,17 +1347,8 @@ do{ assert(!polar && !parametric); \
 	if (auto_array[FIRST_Y_AXIS] & 2)
 	    max_array[FIRST_Y_AXIS] = max_array[SECOND_Y_AXIS];
     }
-#define WRITEBACK(axis,min,max) \
-if(range_flags[axis]&RANGE_WRITEBACK) \
-  {if (auto_array[axis]&1) min = min_array[axis]; \
-   if (auto_array[axis]&2) max = max_array[axis]; \
-  }
 
-    WRITEBACK(FIRST_X_AXIS, xmin, xmax)
-	WRITEBACK(FIRST_Y_AXIS, ymin, ymax)
-	WRITEBACK(SECOND_X_AXIS, x2min, x2max)
-	WRITEBACK(SECOND_Y_AXIS, y2min, y2max)
-	if (strcmp(term->name, "table") == 0)
+    if (strcmp(term->name, "table") == 0)
 	print_table(first_plot, plot_num);
     else {
 	START_LEAK_CHECK();	/* check for memory leaks in this routine */
@@ -1267,6 +1357,27 @@ if(range_flags[axis]&RANGE_WRITEBACK) \
 	do_plot(first_plot, plot_num);
 
 	END_LEAK_CHECK();
+
+        /* after do_plot(), min_array[] and max_array[]
+           contain the plotting range actually used (rounded
+           to tic marks, not only the min/max data values)
+           --> save them now for writeback if requested */
+
+#define SAVE_WRITEBACK(axis) /* ULIG */ \
+  if(range_flags[axis]&RANGE_WRITEBACK) { \
+    set_writeback_min(axis,min_array[axis]); \
+    set_writeback_max(axis,max_array[axis]); \
+  }
+        SAVE_WRITEBACK(FIRST_X_AXIS) 
+        SAVE_WRITEBACK(FIRST_Y_AXIS) 
+        SAVE_WRITEBACK(FIRST_Z_AXIS)
+        SAVE_WRITEBACK(SECOND_X_AXIS)
+        SAVE_WRITEBACK(SECOND_Y_AXIS)
+        SAVE_WRITEBACK(SECOND_Z_AXIS)
+        SAVE_WRITEBACK(T_AXIS)
+        SAVE_WRITEBACK(R_AXIS)
+        SAVE_WRITEBACK(U_AXIS)
+        SAVE_WRITEBACK(V_AXIS)
     }
 
     /* if we get here, all went well, so record this line for replot */
@@ -1365,7 +1476,7 @@ int *plot_num;
 	    /* Ok, fix up the title to include both the xp and yp plots. */
 	    if (xp->title && xp->title[0] != '\0' && yp->title) {
 		tlen = strlen(yp->title) + strlen(xp->title) + 3;
-		new_title = gp_alloc((unsigned long) tlen, "string");
+		new_title = gp_alloc(tlen, "string");
 		strcpy(new_title, xp->title);
 		strcat(new_title, ", ");	/* + 2 */
 		strcat(new_title, yp->title);	/* + 1 = + 3 */
