@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid = "$Id: plot3d.c,v 1.16 1998/12/10 18:30:52 lhecking Exp $";
+static char *RCSid = "$Id: plot3d.c,v 1.16.2.4 2002/01/31 21:18:22 lhecking Exp $";
 #endif
 
 /* GNUPLOT - plot3d.c */
@@ -694,6 +694,17 @@ int pcount;
     int i, curve, surface;
     struct iso_curve *icrvs;
     struct coordinate GPHUGE *points;
+    char *table_format = NULL;
+    char * pcat;
+
+    table_format = gp_alloc(strlen(xformat)+strlen(yformat)+strlen(zformat)+6,
+			    "table format");
+    strcpy(table_format, xformat);
+    strcat(table_format, " ");
+    strcat(table_format, yformat);
+    strcat(table_format, " ");
+    strcat(table_format, zformat);
+    pcat = &table_format[strlen(table_format)];
 
     for (surface = 0, this_plot = first_3dplot; surface < pcount;
 	 this_plot = this_plot->next_sp, surface++) {
@@ -702,12 +713,13 @@ int pcount;
 	curve = 0;
 
 	if (draw_surface) {
+	    strcpy(pcat," %c\n");
 	    /* only the curves in one direction */
 	    while (icrvs && curve < this_plot->num_iso_read) {
 		fprintf(gpoutfile, "\n#IsoCurve %d, %d points\n#x y z type\n",
 			curve, icrvs->p_count);
 		for (i = 0, points = icrvs->points; i < icrvs->p_count; i++) {
-		    fprintf(gpoutfile, "%g %g %g %c\n",
+		    fprintf(gpoutfile, table_format,
 			    points[i].x,
 			    points[i].y,
 			    points[i].z,
@@ -723,6 +735,7 @@ int pcount;
 	if (draw_contour) {
 	    int number = 0;
 	    struct gnuplot_contours *c = this_plot->contours;
+	    strcpy(pcat,"\n");
 	    while (c) {
 		int count = c->num_pts;
 		struct coordinate GPHUGE *p = c->coords;
@@ -732,7 +745,7 @@ int pcount;
 		    /* double blank line to allow plot ... index ... */
 		    fprintf(gpoutfile, "\n# Contour %d, label: %s\n", number++, c->label);
 		for (; --count >= 0; ++p)
-		    fprintf(gpoutfile, "%g %g %g\n", p->x, p->y, p->z);
+		    fprintf(gpoutfile, table_format, p->x, p->y, p->z);
 		/* blank line between segments of same contour */
 		putc('\n', gpoutfile);
 		c = c->next;
@@ -740,6 +753,8 @@ int pcount;
 	}
     }
     fflush(gpoutfile);
+
+    free(table_format);
 }
 
 
@@ -1441,7 +1456,28 @@ if(range_flags[axis]&RANGE_WRITEBACK) \
     else {
 	START_LEAK_CHECK();	/* assert no memory leaks here ! */
 	do_3dplot(first_3dplot, plot_num);
-	END_LEAK_CHECK();
+        END_LEAK_CHECK();
+
+        /* after do_3dplot(), min_array[] and max_array[]
+           contain the plotting range actually used (rounded
+           to tic marks, not only the min/max data values)
+           --> save them now for writeback if requested */
+
+#define SAVE_WRITEBACK(axis) \
+  if(range_flags[axis]&RANGE_WRITEBACK) { \
+    set_writeback_min(axis,min_array[axis]); \
+    set_writeback_max(axis,max_array[axis]); \
+  }
+        SAVE_WRITEBACK(FIRST_X_AXIS) 
+        SAVE_WRITEBACK(FIRST_Y_AXIS) 
+        SAVE_WRITEBACK(FIRST_Z_AXIS) 
+        SAVE_WRITEBACK(SECOND_X_AXIS)
+        SAVE_WRITEBACK(SECOND_Y_AXIS)
+        SAVE_WRITEBACK(SECOND_Z_AXIS)
+        SAVE_WRITEBACK(T_AXIS)
+        SAVE_WRITEBACK(R_AXIS)
+        SAVE_WRITEBACK(U_AXIS)
+        SAVE_WRITEBACK(V_AXIS)
     }
 
     /* if we get here, all went well, so record the line for replot */
@@ -1540,7 +1576,7 @@ int *plot_num;
 		tlen = (xp->title ? strlen(xp->title) : 0) +
 		    (yp->title ? strlen(yp->title) : 0) +
 		    (zp->title ? strlen(zp->title) : 0) + 5;
-		new_title = gp_alloc((unsigned long) tlen, "string");
+		new_title = gp_alloc(tlen, "string");
 		new_title[0] = 0;
 		if (xp->title && xp->title[0] != '\0') {
 		    strcat(new_title, xp->title);
